<?php

use Illuminate\Support\HtmlString;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;

if ( !function_exists('carbonize')) {
    /**
     * Converti recursivement les dates fr en Instance Carbon dans une collection
     * @param  Mixed  $target   Données à traitées
     * @return Mixed            Les données après traitement
     */
    function carbonize($target)
    {
        if ($target instanceof Collection) {
            $target = $target->map(function($item, $key) {
                return carbonize($item);
            });
        } else if (is_object($target)) {
            $datas_array = get_object_vars($target);
            foreach ($datas_array as $key => $value) {
                $target->{$key} = carbonize($value);
            }

        } else if (is_array($target)) {
            foreach ($target as $key => $value) {
               $target[$key] = carbonize($value);
            }
        } else {
            // Detection d'une date selon le format
            if (preg_match("/^[0-9]{1,2}\/[0-9]{1,2}\/[0-9]{4}$/", $target)) {
                $target = \Carbon\Carbon::createFromFormat('d/m/Y', $target)->startOfDay();
            } else if (preg_match("/^[0-9]{1,2}\/[0-9]{1,2}\/[0-9]{2}$/", $target)) {
                $target = \Carbon\Carbon::createFromFormat('d/m/y', $target)->startOfDay();
            }
        }

        return $target;
    }
}

if ( !function_exists('collectize')) {
    /**
     * Converti les tableaux d'une collection en Instance de Collection
     * @param  Mixed  $target   Données à traitées
     * @return Mixed            Les données après traitement
     */     
    function collectize($target)
    {
        if ($target instanceof Collection) {
            $target = $target->map(function($item, $key) {
                return collectize($item);
            });
        } else if (is_object($target)) {
            $datas_array = get_object_vars($target);
            foreach ($datas_array as $key => $value) {
                $target->{$key} = collectize($value);
            }

        } else if (is_array($target)) {

            foreach ($target as $key => $value) {
               $target[$key] = collectize($value);
            }

            $target = collect($target);
        } 

        return $target;
    }
}

if ( !function_exists('objectize')) {
    /**
     * Converti les tableaux associatif d'une collection en stdObject
     * @param  Mixed  $target   Données à traitées
     * @return Mixed            Les données après traitement
     */
    function objectize($target)
    {
        if ($target instanceof Collection) {
            
            $target = $target->map(function($item, $key) {
                return objectize($item);
            });

        } else if (is_object($target)) {
            
            $datas_array = get_object_vars($target);
            foreach ($datas_array as $key => $value) {
                $target->{$key} = objectize($value);
            }

        } else if (is_array($target)) {

            // Détection d'un tableau associatif
            if(count($target) != 0 && array_keys($target) !== range(0, count($target) - 1)) {
                
                $sub_data = new \StdClass;
                foreach ($target as $key => $value) {
                    if (is_array($value)) {
                        $sub_data->{$key} = objectize($value);
                    } else {
                        $sub_data->{$key} = $value;
                    }
                }
                $target = $sub_data;
                
            } else {
                
                foreach ($target as $key => $value) {
                   $target[$key] = objectize($value);
                }
            }

        } 

        return $target;
    }
}

if ( !function_exists('csv_to_collect')) {
    /**
     * Retourne une collection d'object standard à partir d'une chaine de caractère de type csv
     * @param  string $csv       Le contenu du CSV
     * @param  string $delimiter Le séparateur du CSV
     * @return Collection        La collection
     */
    function csv_to_collect(string $csv, string $delimiter = ';') 
    {
        $datas   = array();
        $collect = collect();
        $headers = [];

        // Conversion du string en tableau brut
        $lines = explode(PHP_EOL, $content);
        foreach ($lines as $line => $value) {
            if ($line == 0) {
                $headers = explode($delimiter, $value);
            } else {
                $values = explode($delimiter, $value);
                if (count($values) > 1) {
                    $datas[] = $values;
                }
            }
        }

        // Création de la collection
        foreach ($datas as $line => $data) {

            $obj_line = new stdClass;
            foreach ($headers as $col => $name) {
                
                if (isset($data[$col])) {
                    $col_name = str_replace('-', '_', str_slug($name));
                    $value = trim(trim($data[$col], '"'));
                    $obj_line->{$col_name} = $value;
                }
            }

            $collect->push($obj_line);
        }

        return $collect;
    }
}

if ( !function_exists('array_to_string')) {
    /**
     * Converti un tableau en string
     * @param  array  $array Le tableau à convertir
     * @return string        La chaine de caractères résultante
     */
    function array_to_string(array $array)
    {
        $template = '[%s]';

        $items = '';

        if (count($array)) {
            $items = "'" . implode("', '", $array) . "'";
        }

        return sprintf($template, $items);
    }
}

if ( !function_exists('string_to_float')) {
    /**
     * Parsage d'une chaine de caractère en nombre flottant
     * @param  String $value Valeur à virgule
     * @return Float         Nombre flottant
     */
    function string_to_float(string $value)
    {
        if (!is_null($value)) {
            return floatval(str_replace(',', '.', $value));
        } else {
            return 0;
        }
    }
}

